@@ -7,62 +7,82 @@ import typing as t
 
 import angr
 from angr.analyses.reaching_definitions.dep_graph import DepGraph
-from angr.knowledge_plugins.key_definitions.atoms import Atom
 from angr.knowledge_plugins.functions.function import Function
 from angr.procedures.definitions.glibc import _libc_decls
 
 from pathlib import Path
 
-from . import AnalysisBackend, AnalysisMode, Fuzzability, DEFAULT_SCORE_WEIGHTS
+from . import AnalysisBackend, AnalysisException, Fuzzability, DEFAULT_SCORE_WEIGHTS
 from ..metrics import CallScore
 from ..log import log
 
-# TODO: inherit angr.Analysis
+
 class AngrAnalysis(AnalysisBackend):
     def __init__(
         self,
         target: Path,
-        mode: AnalysisMode,
-        score_weights: t.List[float] = DEFAULT_SCORE_WEIGHTS,
+        include_sym: t.List[str] = [],
+        include_nontop: bool = False,
+        skip_sym: t.List[str] = [],
         skip_stripped: bool = False,
+        score_weights: t.List[float] = DEFAULT_SCORE_WEIGHTS,
     ):
         project = angr.Project(target, load_options={"auto_load_libs": False})
-        super().__init__(project, mode, score_weights)
+        super().__init__(
+            project, include_sym, include_nontop, skip_sym, skip_stripped, score_weights
+        )
 
         log.debug("Doing initial CFG analysis on target")
-        self.cfg = self.target.analyses.CFGFast()
-        _ = self.target.analyses.CompleteCallingConventions(recover_variables=True)
+        self.cfg = self.target.analyses.CFG(
+            resolve_indirect_jumps=True,
+            cross_references=True,
+            force_complete_scan=False,
+            normalize=True,
+            symbols=True,
+        )
 
     def __str__(self) -> str:
         return "angr"
 
     def run(self) -> Fuzzability:
         log.debug("Iterating over functions")
-        for _, func in self.cfg.functions.items():
+        for func in self.cfg.functions.values():
             name = func.name
+            addr = str(hex(func.addr))
 
+            # just in case repeats show up again
             if name in self.visited:
                 continue
             self.visited += [name]
 
-            if self.mode == AnalysisMode.RECOMMEND and self.skip_analysis(func):
+            log.debug(f"Checking if we should ignore {name}")
+            if self.skip_analysis(func):
                 log.warning(f"Skipping {name} from fuzzability analysis.")
-                self.skipped[name] = str(hex(func.addr))
+                self.skipped[name] = addr
                 continue
 
-            # if recommend mode, filter and run only those that are top-level
-            if self.mode == AnalysisMode.RECOMMEND and not self.is_toplevel_call(func):
-                self.skipped[name] = str(hex(func.addr))
+            log.debug(f"Checking to see if {name} is a top-level call")
+            if not self.include_nontop and not self.is_toplevel_call(func):
+                log.warning(
+                    f"Skipping {name} (not top-level) from fuzzability analysis."
+                )
+                self.skipped[name] = addr
                 continue
 
-            log.info(f"Conducting fuzzability analysis on function symbol '{name}'")
+            log.info(f"Starting analysis for function {name}")
             score = self.analyze_call(name, func)
             self.scores += [score]
 
+        if len(self.scores) == 0:
+            raise AnalysisException(
+                "No suitable function symbols filtered for analysis."
+            )
+
         return super()._rank_fuzzability(self.scores)
 
     def analyze_call(self, name: str, func: Function) -> CallScore:
         stripped = "sub_" in name
+        addr = str(hex(func.addr))
 
         # no need to check if no name available
         # TODO: maybe we should run this if a signature was recovered
@@ -73,7 +93,7 @@ class AngrAnalysis(AnalysisBackend):
 
         return CallScore(
             name=name,
-            loc=str(hex(func.addr)),
+            loc=addr,
             toplevel=self.is_toplevel_call(func),
             fuzz_friendly=fuzz_friendly,
             risky_sinks=self.risky_sinks(func),
@@ -86,6 +106,9 @@ class AngrAnalysis(AnalysisBackend):
     def skip_analysis(self, func: Function) -> bool:
         name = func.name
 
+        if super().skip_analysis(name):
+            return True
+
         # ignore imported functions or syscalls
         if func.is_syscall:
             return True
@@ -94,14 +117,6 @@ class AngrAnalysis(AnalysisBackend):
         if name in _libc_decls:
             return True
 
-        # ignore instrumentation
-        # if name.startswith("__"):
-        #    return True
-
-        # ignore runtime calls from the binary
-        if name in ["_init", "frame_dummy", "call_weak_fn", "$x", "_fini"]:
-            return True
-
         # if set, ignore all stripped functions for faster analysis
         if "Unresolvable" in name:
             return True
@@ -109,39 +124,47 @@ class AngrAnalysis(AnalysisBackend):
         return False
 
     def is_toplevel_call(self, target: Function) -> bool:
-        """
-        TODO: make this work with less of a performance downgrade.
-        """
-        cfg_nodes = self.cfg.get_all_nodes(target.addr)
-        if len(cfg_nodes) == 0:
-            return True
-        # return self.cfg.get_predecessors(cfg_nodes[0]) == 0
-        return True
-
-    def risky_sinks(self, func: Function) -> int:
-        """
-        TODO: really slow
-        """
-        log.debug(f"{func.name} - checking for risky sinks")
-
-        target = self.target.kb.functions.function(name=func.name)
-        program_rda = self.target.analyses.ReachingDefinitions(
-            subject=target, dep_graph=DepGraph()
+        return (
+            len(set(target._function_manager.callgraph.predecessors(target.addr))) == 0
         )
 
-        """
-        for sink_name in ["memcpy", "strcpy", ""]:
-            sink_function = self.target.kb.functions.function(name=sink_name)
+    def risky_sinks(self, func: Function) -> int:
+        log.debug(f"{func.name} - checking for risky sinks")
 
-            parameter_position = 1
-            parameter_type = _libc_decls[sink_name].args[parameter_position]
-            parameter_atom = Atom.from_argument(
-                sink_function.calling_convention.arg_locs()[parameter_position],
-                self.target.arch.registers
+        """
+        # TODO: do an interprocedural analysis starting from the function
+        func_cfg = self.target.analyses.CFGFast(
+            start_at_entry=False,
+            function_starts=[func.addr]
+        )
+        """
+
+        risky_sinks = 0
+        for cs in func.get_call_sites():
+            insn = func.get_call_target(cs)
+            call_site = self.cfg.kb.functions.function(addr=insn)
+
+            # TODO: should we traverse further if not a imported func
+            if AngrAnalysis._is_risky_call(call_site.name):
+                risky_sinks += 1
+
+            """
+            rd = self.target.analyses.ReachingDefinitions(
+                subject=func,
+                func_graph=func.graph,
+                cc=func.calling_convention,
+                observation_points=[
+                    (
+                        "insn",
+                        insn,
+                        0,
+                    )
+                ],
+                dep_graph=DepGraph(),
             )
-        """
+            """
 
-        return len(func.get_call_sites())
+        return risky_sinks
 
     def get_coverage_depth(self, target: Function) -> int:
         """
