@@ -6,7 +6,6 @@ config.py
 """
 import typing as t
 
-from os.path import dirname, abspath
 from pathlib import Path
 
 
@@ -19,6 +18,17 @@ def get_project_root() -> Path:
 # TODO: we should do a very initial parse on the file to determine if it is C++ source
 SOURCE_FILE_EXTS: t.List[str] = [".c", ".cpp", ".cc", ".cp" ".cxx", ".h", ".hpp", ".hh"]
 
+GLOBAL_IGNORES: t.List[str] = [
+    "__cxa_finalize",
+    "__gmon_start__",
+    "_init",
+    "_fini",
+    "frame_dummy",
+    "call_weak_fn",
+    "register_tm_clones",
+    "$x",
+]
+
 # Interesting symbol name patterns to check for fuzzable
 INTERESTING_PATTERNS: t.List[str] = [
     # Consuming Inputs
@@ -31,6 +41,12 @@ INTERESTING_PATTERNS: t.List[str] = [
     # Decryption Routines
     "encode",
     "decode",
+    # Other stuff
+    "draw",
+    "image",
+    "img",
+    "load",
+    "url",
 ]
 
 # Function name patterns that include INTERESTING_PATTERNS but
